/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you want to add, delete, or rename functions or slots, use
** Qt Designer to update this file, preserving your code.
**
** You should not define a constructor or destructor in this file.
** Instead, write your code in functions called init() and destroy().
** These will automatically be called by the form's constructor and
** destructor.
*****************************************************************************/
#include <qxml.h>
#include <qtextbrowser.h>
#include <qstatusbar.h>
#include "ananas.h"
#include "mainform.h"
#include "alog.h"

extern MainForm *mainform;
extern dASA *asaform=NULL;


/*!
* Init Ananas sntax assistant (ASA)
*/
void
dASA::init()
{
	delete statusBar();
	asaform=this;
	QString filePath = "";
	articlelist->header()->hide();
	foundList->header()->hide();
	bFrvd->setEnabled(FALSE);
	bBack->setEnabled(FALSE);
	connect( textArea, SIGNAL( backwardAvailable(bool) ), bBack, SLOT( setEnabled(bool) ) );
	connect( textArea, SIGNAL( forwardAvailable(bool) ), bFrvd, SLOT( setEnabled(bool) ) );
	loadASA();
}

/*!
* Destrty Ananas sntax assistant (ASA)
*/
void dASA::destroy()
{
	asaform = 0;
}


/*!
* Find text in keywords 
*/
void dASA::findThis_textChanged( const QString &str )
{

	QListViewItem *farticle, *article, *group, *g, *a;
	QString keywords;
	int i;
	QListViewItemIterator it( articlelist);
	if( str.length() > 3 )
	{
		foundList->clear();
		while ( it.current() )
		{
			if (it.current()->text(3) && it.current()->text(3) != "")
			{
				keywords = it.current()->text(3);
				keywords.append(it.current()->text(0));
				i = keywords.findRev( str, -1,FALSE );
				if (i >= 0 )
				{
					farticle = new QListViewItem(foundList);
					farticle->setText(0,it.current()->text(0));
					farticle->setText(1,it.current()->text(2));
				}
			}
			++it;
		}
	}
}


void dASA::findButton_clicked()
{

}


void dASA::articlelist_doubleClicked( QListViewItem * )
{

}


void dASA::findIndex_textChanged( const QString & )
{

}


/*!
* Load Ananas sntax assistant (ASA)
* Find structure in /usr/share/ananas/asa/
* Description plased in *.conf files
* Example^
* cat /usr/share/ananas/asa/Widgets/acombobox.conf 
* <?xml version="1.0" encoding="iso-8859-1"?>
* <module>
*   <group>Widgets</group>
*   <name>aCombobox</name>
*   <icon>acombobox.png</icon>
*   <file>aCombobox.html</file>
*   <keys>aComboBox acombobox combo</keys>
*   <author>Dmitriy Kruflikov</author>
* </module>
*
* Plase icon (eg. acombobox.png) it this catalogue
*
*/
void dASA::loadASA()
{
	articlelist->clear();
	int i, j;
	QString asaDir, asaFile;
#ifdef Q_OS_WIN32
	asaDir = qApp->applicationDirPath()+"/asa/";
#else
	asaDir = "/usr/share/ananas/asa/";
#endif
	asaFile = "*.conf";
	QDir cur_dir, dir = QDir(asaDir);
	QStringList dirs = dir.entryList(QDir::Dirs| QDir::Hidden | QDir::NoSymLinks );
	QStringList foundFiles;
	QStringList files;
	QString asaPath = "";
	for (j =0; j < dirs.size(); ++j)
	{
		cur_dir = QDir(dirs[j]);
		asaPath = asaDir+dirs[j];
		cur_dir = QDir(asaPath);

		files = cur_dir.entryList(asaFile, QDir::Files);
		for ( i = 0; i < files.size(); ++i )
		{
			QFile file(cur_dir.filePath(files[i]));
			filePath = asaPath;
			DomParser(&file);
			foundFiles << files[i];
		}
	}
	articlelist->setRootIsDecorated ( TRUE );
}



void dASA::DomParser( QIODevice *device)
{

	QString errorStr;
	int i;
	QString findres = "No";
	QDomDocument doc;
	QListViewItem *article;
	QListViewItem *group = articlelist->firstChild();


	if (!doc.setContent(device, true)) {
		qDebug("Error");
		return;
	}

	QDomElement root = doc.documentElement();
	if (root.tagName() != "module") {
		qWarning("The file is not a module file");
		return;
	}

	for (i = 0; i < articlelist->childCount(); ++i)
	{
		// Find group, if exist or create, if no
		if (articlelist->childCount() != 0 && articlelist->findItem(root.firstChild().toElement().text(), 0 ) )
		{
			group = articlelist->findItem(root.firstChild().toElement().text(), 0 );
			article = new QListViewItem(group);
			findres = "Yes";
			break;
		}
	}
	if (articlelist->childCount() == 0 || findres != "Yes"){
		group = new QListViewItem(articlelist);
		article = new QListViewItem(group);
	}
	group->setText(0, root.firstChild().toElement().text());


	QDomNode node = root.firstChild();
	while (!node.isNull())
	{
		if (node.toElement().tagName() == "name")
		{
			article->setText(0, node.toElement().text());
		}
		if (node.toElement().tagName() == "file") {
			article->setText(1, node.toElement().text());
			article->setText(2, filePath +"/"+ node.toElement().text());
		}
		if (node.toElement().tagName() == "keys") {
			article->setText(3, node.toElement().text());
		}
		if (node.toElement().tagName() == "author") {
			article->setText(5, node.toElement().text());
		}
		if (node.toElement().tagName() == "icon") {
			article->setPixmap(0, filePath +"/"+node.toElement().text());
		}
		node = node.nextSibling();
	}
	group->setExpandable(TRUE);
}


void dASA::articlelist_selectionChanged()
{
	labelAuthor->setText(articlelist->currentItem()->text(5));
	textArea->setSource(articlelist->currentItem()->text(2));
}


void dASA::bFrvd_clicked()
{
	textArea->forward();
}


void dASA::bBack_clicked()
{
	textArea->backward();
}


void dASA::foundList_clicked( QListViewItem *item )
{
	textArea->setSource(item->text(1));
}


void dASA::textArea_forwardAvailable( bool fa)
{
	bFrvd->setEnabled(fa);
}


void dASA::textArea_backwardAvailable( bool ba)
{
	bBack->setEnabled(ba);
}

/*!
 * \en	Find a string and open Assistant window. \_en
 * \ru 	Ищет строку и открывает окно Ассистента.\_ru
 * 	\param f - \en string \_en \ru строка для поиска \_ru
 */
void dASA::find( const QString &f )
{
	if (f != "")
	{
		tabWidget->setCurrentPage(2);
		findThis->setText( f );
		findThis_textChanged( f );
	}else{
		tabWidget->setCurrentPage(0);
	}
}
